/* Integrating DJ Native Swing into NetBeans RCP
 *
 * See the DJ Project by Christopher Deckers
 * http://djproject.sourceforge.net/ns/index.html
 *
 * Geertjaan Wielenga "Step-by-Step Instructions for Integrating DJ Native
 * Swing into NetBeans RCP"
 * http://netbeans.dzone.com/how-to-nb-djnative-swing?utm_source=feedburner&utm_
 * medium=feed&utm_campaign=Feed%3A+zones%2Fnetbeans+%28NetBeans+Zone%29
 *
 * Eclipse swt library
 * http://download.eclipse.org/eclipse/downloads/drops/R-3.6-201006080911/
 *
 * This code may be freely distributed and modified under the
 * terms of the GNU Lesser General Public Licence v2.1.
 */
package org.starbeans.modules.nativeswing;

import java.net.URL;
import java.util.Collection;
import org.openide.awt.HtmlBrowser.URLDisplayer;
import org.openide.util.Lookup;
import org.openide.util.Mutex;
import org.openide.util.lookup.ServiceProvider;

/**
 *
 * @author George Waldon
 */
/* DISABLE TO ENABLE GENERAL WEB BROWSER OPTIONS */
/* SEE ALSO LAYER */
@ServiceProvider(service=URLDisplayer.class,position=100)
public class DJNSURLDisplayer extends URLDisplayer {
    
    private boolean initError;

    @Override
    public void showURL(final URL u) {
        if(u==null)
            return;
        
        Mutex.EVENT.readAccess(new Runnable() {
            @Override
            public void run() {
                if(initError) {
                    showURLExternal(u);
                    return;
                }
                try {
                    if (!NativeBrowserComponent.ShowBrowserWindow(u)) {
                        showURLExternal(u);
                    }
                }
                // If any error or exception is catched, default browsing to
                // external viewing.
                catch (Throwable er) {
                    initError = true;
                    showURLExternal(u);
                }
            }
        });
    }
    
    @Override
    public void showURLExternal(final URL u) {
        Lookup.Result<URLDisplayer> result = Lookup.getDefault().lookupResult(URLDisplayer.class);
        Collection<? extends URLDisplayer> all = result.allInstances();
        for(URLDisplayer ud:all) {
            if(ud!=this) {
                ud.showURLExternal(u);
                break;
            }
        }
    }    
}
