/*
 * @author George Waldon
 *
 * This code may be freely distributed and modified under the
 * terms of the GNU Lesser General Public Licence v2.1.
 */
package org.starbeans.modules.nativeswing;

import chrriis.dj.nativeswing.swtimpl.components.JWebBrowser;
import chrriis.dj.nativeswing.swtimpl.components.WebBrowserCommandEvent;
import chrriis.dj.nativeswing.swtimpl.components.WebBrowserEvent;
import chrriis.dj.nativeswing.swtimpl.components.WebBrowserListener;
import chrriis.dj.nativeswing.swtimpl.components.WebBrowserNavigationEvent;
import chrriis.dj.nativeswing.swtimpl.components.WebBrowserWindowOpeningEvent;
import chrriis.dj.nativeswing.swtimpl.components.WebBrowserWindowWillOpenEvent;
import java.awt.BorderLayout;
import java.awt.Image;
import java.net.URL;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.openide.util.ImageUtilities;
import org.openide.util.NbBundle;
import org.openide.windows.Mode;
import org.openide.windows.TopComponent;
import org.openide.windows.WindowManager;

public class NativeBrowserComponent extends TopComponent {

    private static final Logger LOGGER = Logger.getLogger(NativeBrowserComponent.class.getName());

    private static final String PREFERRED_ID = "nativebrowser"; //NOI18N

    private transient JWebBrowser browser;
    private transient BrowserListener listener;

    /** Show a URL in an swt browser window.
     *
     * @param url The url to reach.
     * @param title The title of the TopComponent.
     * @param icon The icon to display in the title bar.
     * @return true if a BrowserProvider has been found to display the url, otherwise false.
     */
    public static boolean ShowBrowserWindow(URL url, String title, Image icon) {
        DJNSFactory.initialize();
        WindowManager wm = WindowManager.getDefault();
        Mode mode = wm.findMode("editor"); //NOI18N
        if (mode == null) {
            //See doc, editor mode should always exists.
            return false;
        }
        NativeBrowserComponent bw = null;
        try {
            bw = new NativeBrowserComponent(url, title, icon);
        } catch (Exception ex) {
            // Most likely missing library
            LOGGER.log(Level.INFO, "Error: Cannot create browser window", ex); //NOI18N
            return false;
        }
        bw.open();
        bw.requestActive();
        return true;
    }

    public static boolean ShowBrowserWindow(URL url) {
        return ShowBrowserWindow(url, null, null);
    }

    public NativeBrowserComponent(URL url, String title, Image icon) {
        setLayout(new BorderLayout());
        String name = (title != null && !title.isEmpty()) ? title : NbBundle.getMessage(NativeBrowserComponent.class, "LBL_swtbrowser");
        setName(name);
        Image img = icon;
        if (img == null) {
            img = ImageUtilities.loadImage("org/starbeans/modules/nativeswing/resources/webicon.png"); //NOI18N
        }
        setIcon(img);

        browser = DJNSFactory.createJWebBrowser();
        add(browser, BorderLayout.CENTER);
        listener = new BrowserListener();
        browser.addWebBrowserListener(listener);
        browser.setMenuBarVisible(false);
        browser.setStatusBarVisible(false);
        if (url != null) {
            browser.navigate(url.toString());
        }
    }

    @Override
    protected String preferredID() {
        return PREFERRED_ID;
    }

    @Override
    public int getPersistenceType() {
        return TopComponent.PERSISTENCE_NEVER;
    }
    
    @Override
    protected void componentClosed() {
        super.componentClosed();
        browser.removeWebBrowserListener(listener);
        browser.disposeNativePeer();
    }
    
    private class BrowserListener implements WebBrowserListener {

        public BrowserListener() {
        }

        @Override
        public void windowWillOpen(WebBrowserWindowWillOpenEvent wbwwoe) {
        }

        @Override
        public void windowOpening(WebBrowserWindowOpeningEvent wbwoe) {
        }

        @Override
        public void windowClosing(WebBrowserEvent wbe) {
        }

        @Override
        public void locationChanging(WebBrowserNavigationEvent wbne) {
        }

        @Override
        public void locationChanged(WebBrowserNavigationEvent wbne) {
        }

        @Override
        public void locationChangeCanceled(WebBrowserNavigationEvent wbne) {
        }

        @Override
        public void loadingProgressChanged(WebBrowserEvent wbe) {
        }

        @Override
        public void titleChanged(WebBrowserEvent wbe) {
            JWebBrowser jb = (JWebBrowser)wbe.getSource();
            String title = jb.getPageTitle();
            if(title!=null && !title.isEmpty()) {
                if(title.length()>24)
                    title = title.substring(0, 24) + "..."; //NOI18N
                NativeBrowserComponent.this.setDisplayName(title);
            }
        }

        @Override
        public void statusChanged(WebBrowserEvent wbe) {
        }

        @Override
        public void commandReceived(WebBrowserCommandEvent wbce) {
        }
    }
    
    
}
