/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
 *
 * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
 * Other names may be trademarks of their respective owners.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or sbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * sbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 *
 * Portions Copyrighted 2008-2014 George Waldon
 */

package org.starbeans.modules.corelite.actions;

import java.awt.Component;
import java.awt.EventQueue;
import java.awt.event.ActionEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.IOException;
import java.lang.ref.WeakReference;
import java.util.Iterator;
import java.util.Set;
import java.util.logging.Logger;
import javax.swing.Action;
import javax.swing.JComponent;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import org.openide.actions.NewTemplateAction;
import org.openide.awt.Actions;
import org.openide.awt.DynamicMenuContent;
import org.openide.explorer.view.MenuView;
import org.openide.loaders.DataFilter;
import org.openide.loaders.DataFolder;
import org.openide.loaders.DataNode;
import org.openide.loaders.DataObject;
import org.openide.loaders.DataShadow;
import org.openide.loaders.TemplateWizard;
import org.openide.nodes.*;
import org.openide.util.Exceptions;
import org.openide.util.Mutex;
import org.openide.util.HelpCtx;
import org.openide.util.Lookup;
import org.openide.util.LookupListener;
import org.openide.util.NbBundle;
import org.openide.util.Utilities;
import org.openide.util.actions.*;
import org.openide.windows.TopComponent;
import org.openide.windows.WindowManager;
import org.starbeans.modules.corelite.utilities.UIUtilities;

/** Creates a new object from template in the selected folder. 
 * This variant of NewTemplateAction inlines the first template folder. 
 * This is convenient when combined with a Context2Action that generates new 
 * objects of other kinds.
 *
* @see DataObject#isTemplate
*
* @author Petr Hamernik, Dafe Simonek, George Waldon
*/
// From org.openide.actions.NewTemplateAction in openide.loaders
public class NewTemplateInlineAction extends NodeAction {

    private static DataObject selectedTemplate;
    private static DataFolder targetFolder;

    /** Maximum count of recent templates. */
    private static int MAX_RECENT_ITEMS = 5;
    
    /** Getter for wizard.
     * @param the node that is currently activated
     * @return the wizard or null if the wizard should not be enabled
    */
    static TemplateWizard getWizard (Node n) {
        if (n == null) {
            Node[] arr = WindowManager.getDefault ().getRegistry ().getActivatedNodes ();
            if (arr.length == 1) {
                n = arr[0];
            }
        }

        // if activated node isn't folder try parent which should be folder
        Node folder = n;
        // bugfix #29661, start finding the target folder with null folder
        targetFolder = null;
        while (targetFolder == null && folder != null) {
            targetFolder = folder.getCookie(DataFolder.class);
            folder = folder.getParentNode ();
        }
        
        NewTemplateAction.Cookie c = n == null ? null : n.getCookie(NewTemplateAction.Cookie.class);
        if (c != null) {
            TemplateWizard t = c.getTemplateWizard ();
            if (t != null) {
                return t;
            }
        }

        return new DefaultTemplateWizard();
    }

    private boolean active = false;

    // This method is called only for the File->New menu item
    // it gets the node selection from the active TC
    @Override
    protected void performAction (Node[] activatedNodes) {
    }
    
    @Override
    protected boolean asynchronous() {
        return true;
    }

    /* Enables itself only when activates node is DataFolder.
    */
    protected boolean enable (Node[] activatedNodes) {
        if ((activatedNodes == null) || (activatedNodes.length != 1))
            return false;

        NewTemplateAction.Cookie c = activatedNodes[0].getCookie(NewTemplateAction.Cookie.class);
        if (c != null) {
            // if the current node provides its own wizard...
            return c.getTemplateWizard () != null;
        }
        
        DataFolder cookie = activatedNodes[0].getCookie(DataFolder.class);
        if (cookie != null && cookie.getPrimaryFile ().canWrite ()) {
            return true;
        }
        return false;
    }

    public String getName() {
        return NbBundle.getMessage(DataObject.class, "NewTemplate");
    }

    public HelpCtx getHelpCtx() {
        return new HelpCtx (NewTemplateInlineAction.class);
    }
    
    @Override
    public JMenuItem getMenuPresenter () {
        return new Actions.MenuItem (this, true) {
            @Override
                   public void setEnabled (boolean e) {
                       super.setEnabled (true);
            }
        };
    }
    
    @Override
    public Component getToolbarPresenter () {
        return new Actions.ToolbarButton (this) {
            @Override
                   public void setEnabled (boolean e) {
                       super.setEnabled (true);
            }
        };
    }
    
    /* Creates presenter that displayes submenu with all
    * templates.
    */
    @Override
    public JMenuItem getPopupPresenter() {
        return getPopupPresenter (null, this);
    }
    
    /*STARBEANS*/
    protected JMenuItem getPopupPresenter (final Lookup actionContext, Action action) {
        Node[] nodes = new Node[0];
        //if (actionContext != null) {
            nodes = getNodesFromLookup (actionContext);
        //}
        final Node n = (nodes.length == 1) ? nodes[0] : null;
        JMenuItem menu = new Inline(n,actionContext);
        Actions.connect (menu, action, true);
        return menu;
    }
    
    
    /** Inline menu.
     */
    private static final class Inline extends JMenuItem implements DynamicMenuContent {
        
        Node n;
        Lookup actionContext;
        
        Inline(Node n, Lookup actionContext) {
            super();
            this.n = n;
            this.actionContext = actionContext;
        }

        public JComponent[] getMenuPresenters() {
            return synchMenuPresenters(new JComponent[0]);
        }  
        
        public JComponent[] synchMenuPresenters(JComponent[] items) {
            Node root = getTemplateRoot (n);

            //hack in waiting for more available time! regenerate the nodes
            new RootChildren (null).getRootFolder().getNodeDelegate ().getChildren ().getNodes (true);

            Node[] nodes = root.getChildren().getNodes(true);
            JComponent[] comps = new JComponent[nodes.length];
            for(int i = 0; i<nodes.length;i++) {
                final Node fnode = nodes[i];
                JMenuItem menu = new MenuView.Menu(null, new TemplateActionListener(actionContext), false) {
                    // this is the only place MenuView.Menu needs the node ready
                    // so lets prepare it on-time
                    @Override
                    public JPopupMenu getPopupMenu() {
                        if (node == null) node = fnode;
                        return super.getPopupMenu();
                    }
                };
                //Don't draw these folder icons
                //menu.setIcon(new ImageIcon(fnode.getIcon(java.beans.BeanInfo.ICON_COLOR_16x16)));
                menu.setText(fnode.getDisplayName());
                comps[i] = menu;
            }
            return comps;      
        }
    }

    /** Create a hierarchy of templates.
    * @return a node representing all possible templates
    */
    public static Node getTemplateRoot () {
        RootChildren ch = new RootChildren (null);
        // create the root
        return ch.getRootFolder ().new FolderNode (ch);
    }
    
    private static Node getTemplateRoot (Node n) {
        RootChildren ch = new RootChildren (n);
        // create the root
        Node help = ch.getRootFolder ().new FolderNode (ch);
        return help;
    }
    
//STARBEANS, already in NewTemplateAction
//    /** Cookie that can be implemented by a node if it wishes to have a 
//     * special templates wizard.
//     */
//    public static interface Cookie extends Node.Cookie {
//        /** Getter for the wizard that should be used for this cookie.
//         */
//        public TemplateWizard getTemplateWizard ();
//    }
    
    /** Checks whether an object is acceptable for display as a container.
     */
    private static boolean acceptObj (DataObject obj) {
        if (obj.isTemplate ()) {
            return true;
        }

        if (obj instanceof DataFolder) {
            Object o = obj.getPrimaryFile ().getAttribute ("simple"); // NOI18N
            return o == null || Boolean.TRUE.equals (o);
        }

        return false;
        
    }


    /** Actions listener which instantiates the template */
    /*private*/static class TemplateActionListener implements NodeAcceptor, DataFilter {
        static final long serialVersionUID =1L;
        Lookup actionContext;
        TemplateActionListener(Lookup context) {
            actionContext = context;
        }
        public boolean acceptNodes (Node[] nodes) {
            Node[] nodesInContext = null;
            /*STARBEANS*/
            //if (actionContext != null) {
                nodesInContext = getNodesFromLookup(actionContext);
                
            //}
            if ((nodesInContext == null) || (nodesInContext.length != 1)) {
                Logger.getAnonymousLogger().warning("Wrong count of nodes in context lookup."); //NOI18N
                    return false;
            }
            if ((nodes == null) || (nodes.length != 1)) {
                Logger.getAnonymousLogger().warning("Wrong count of selected nodes in popup menu."); //NOI18N
                return false;
            }
            Node n = nodes[0];
            DataObject obj = n.getCookie(DataObject.class);
            if (obj == null || !obj.isTemplate ()) {
                Logger.getAnonymousLogger().warning("Selected node in popup menu is not acceptable."); //NOI18N
                // do not accept
                return false;
            }
            
            // bugfix #38421, read node in contextLookup to select the right wizard
            TemplateWizard wizard = getWizard (nodesInContext[0]);
            
            try {
                wizard.setTargetName (null);
                //STARBEANS: open new file
                Set<DataObject> resultSet = wizard.instantiate (obj, targetFolder);
                if (resultSet == null || resultSet.isEmpty()) {
                    // no new object, no work
                    return true;
                }
                Iterator<DataObject> it = resultSet.iterator();
                while (it.hasNext()) {
                    DataObject newDO = it.next();
                    if (newDO != null) {
                        UIUtilities.openAndSelectNewObject(newDO);
                        break;
                    }
                }

            } catch (IOException e) {
                Exceptions.attachLocalizedMessage(e,
                                                  org.openide.util.NbBundle.getMessage(org.openide.loaders.DataObject.class,
                                                                                       "EXC_TemplateFailed"));
                Exceptions.printStackTrace(e);
            }

            // ok
            return true;
        }

        /** Data filter impl.
        */
        public boolean acceptDataObject (DataObject obj) {
            return acceptObj (obj);
        }
    }
    
    /** Root template childen.
     */
    private static class RootChildren extends Children.Keys<Node>
    implements NodeListener {
        /** last wizard used with the root */
        private TemplateWizard wizard;
        /** Folder of templates */
        private DataFolder rootFolder;
        /** node to display templates for or null if current selection
         * should be followed
         */
        private WeakReference<Node> current;
        /** weak listener */
        private NodeListener listener = org.openide.nodes.NodeOp.weakNodeListener (this, null);
        
        /** Instance not connected to any node.
         */
        public RootChildren (Node n) {
            TopComponent.Registry reg = WindowManager.getDefault ().getRegistry ();
            reg.addPropertyChangeListener (org.openide.util.WeakListeners.propertyChange (this, reg));
            
            updateWizard (getWizard (n));
        }
        
        public DataFolder getRootFolder () {
            if (rootFolder == null) {
                // if rootFolder is null then initialize folder
                doSetKeys ();
            }
            return rootFolder;
        }
               

        /** Creates nodes for nodes.
         */
        protected Node[] createNodes(Node n) {
            String nodeName = n.getDisplayName();
            
            DataObject obj = null;
            DataShadow shadow = n.getCookie(DataShadow.class);
            if (shadow != null) {
                // I need DataNode here to get localized name of the
                // shadow, but without the ugly "(->)" at the end
                DataNode dn = new DataNode(shadow, Children.LEAF);
                nodeName = dn.getDisplayName();
                obj = shadow.getOriginal();
                n = obj.getNodeDelegate();
            }
            
            if (obj == null)
                obj = n.getCookie(DataObject.class);
            if (obj != null) {
                if (obj.isTemplate ()) {
                    // on normal nodes stop recursion
                    return new Node[] { new DataShadowFilterNode (n, LEAF, nodeName) };
                }
            
                if (acceptObj (obj)) {
                    // on folders use normal filtering
                    return new Node[] { new DataShadowFilterNode (n, new TemplateChildren (n), nodeName) };
                }
            }
            
            return null;
        }
        
        /** Check whether the node has not been updated.
         */
        private void updateNode (Node n) {            
            if (current != null && current.get () == n) {
                return;
            }
            
            Node prev = current != null? current.get(): null;
            if (prev != null) {
                prev.removeNodeListener (listener);
            }
            
            n.addNodeListener (listener);
            current = new WeakReference<Node> (n);
        }
        
        /** Check whether the wizard was not updated.
         */
        private void updateWizard (TemplateWizard w) {
            if (wizard == w) {
                return;
            }
            
            if (wizard != null) {
                Node n = wizard.getTemplatesFolder ().getNodeDelegate ();
                n.removeNodeListener (listener);
            }
            
            Node newNode = w.getTemplatesFolder ().getNodeDelegate ();
            newNode.addNodeListener (listener);
            wizard = w;
            
            updateKeys ();
        }
        
        /** Updates the keys.
         */
        private void updateKeys () {
            // updateKeys can be called while holding Children.MUTEX
            //   --> replan getNodes(true) to a new thread
            TemplateNodeUtils.reqProcessor().post(new Runnable() {
                @Override
                public void run() {
                    doSetKeys ();
                }
            });
        }
        
        // don't call this while holding Children.MUTEX
        private void doSetKeys () {
            rootFolder = wizard.getTemplatesFolder ();
            setKeys (rootFolder.getNodeDelegate ().getChildren ().getNodes (true));
        }
         
         /** Fired when the order of children is changed.
        /** Fired when the order of children is changed.
         * @param ev event describing the change
         */
        public void childrenReordered(NodeReorderEvent ev) {
            updateKeys ();
        }        
        
        /** Fired when a set of children is removed.
         * @param ev event describing the action
         */
        public void childrenRemoved(NodeMemberEvent ev) {
            updateKeys ();
        }
        
        /** Fired when a set of new children is added.
         * @param ev event describing the action
         */
        public void childrenAdded(NodeMemberEvent ev) {
            updateKeys ();
        }
        
        /** Fired when the node is deleted.
         * @param ev event describing the node
         */
        public void nodeDestroyed(NodeEvent ev) {
        }

        /** Listen on changes of cookies.
         */
        public void propertyChange(java.beans.PropertyChangeEvent ev) {
            String pn = ev.getPropertyName ();
            
            if (current != null && ev.getSource () == current.get ()) {
                // change in current node
                if (Node.PROP_COOKIE.equals (pn)) {
                    final Node node = current.get();
                    Mutex.EVENT.readAccess(new Runnable() {
                        public void run() {
                            updateWizard (getWizard (node));
                        }
                    });
                }
            } else {
                // change in selected nodes
                if (TopComponent.Registry.PROP_ACTIVATED_NODES.equals (pn)) {
                    // change the selected node
                    Node[] arr = WindowManager.getDefault ().getRegistry ().getActivatedNodes ();
                    if (arr.length == 1) {
                        // only if the size is 1
                        updateNode (arr[0]);
                    }
                }
            }
        }
        
    }
    
    /** Filter node children, that stops on data objects (does not go futher)
    */
    private static class TemplateChildren extends FilterNode.Children {
        public TemplateChildren (Node or) {
            super (or);
        }
        
        /** Creates nodes for nodes.
         */
        @Override
        protected Node[] createNodes(Node n) {
            if (EventQueue.isDispatchThread()) {
                return new Node[] { new DataShadowFilterNode(n, LEAF, n.getDisplayName()) };
            }
            
            String nodeName = n.getDisplayName();
            
            DataObject obj = null;
            DataShadow shadow = n.getCookie(DataShadow.class);
            if (shadow != null) {
                // I need DataNode here to get localized name of the
                // shadow, but without the ugly "(->)" at the end
                DataNode dn = new DataNode(shadow, Children.LEAF);
                nodeName = dn.getDisplayName();
                obj = shadow.getOriginal();
                n = obj.getNodeDelegate();
            }
            
            if (obj == null)
                obj = n.getCookie(DataObject.class);
            if (obj != null) {
                if (obj.isTemplate ()) {
                    // on normal nodes stop recursion

                }
            
                if (acceptObj (obj)) {
                    // on folders use normal filtering
                    return new Node[] { new DataShadowFilterNode (n, new TemplateChildren (n), nodeName) };
                }
            }
            return new Node[] { new DataShadowFilterNode(n, LEAF, nodeName) };
        }

        final void refreshKeyImpl(Node n) {
            refreshKey(n);
        }

    }

    private static class DataShadowFilterNode extends FilterNode implements NodeListener {
        
        private String name;
        
        public DataShadowFilterNode (Node or, org.openide.nodes.Children children, String name) {
            super (or, children);
            this.name = name;
            disableDelegation(FilterNode.DELEGATE_SET_DISPLAY_NAME);
            or.addNodeListener(NodeOp.weakNodeListener(this, or));
                    
        }
        
        @Override
        public String getDisplayName() {
            return name;
        }

        private void refresh() {
            Node n = getOriginal();
            String nodeName = n.getDisplayName();

            DataObject obj = null;
            DataShadow shadow = n.getCookie(DataShadow.class);
            if (shadow != null) {
                // I need DataNode here to get localized name of the
                // shadow, but without the ugly "(->)" at the end
                DataNode dn = new DataNode(shadow, Children.LEAF);
                nodeName = dn.getDisplayName();
                obj = shadow.getOriginal();
                n = obj.getNodeDelegate();
            }

            if (obj == null) {
                obj = n.getCookie(DataObject.class);
            }
            if (obj != null) {
                if (obj.isTemplate()) {
                    // on normal nodes stop recursion
                    this.name = nodeName;
                    this.setChildren(Children.LEAF);
                    return;
                }

                if (acceptObj(obj)) {
                    // on folders use normal filtering
                    this.name = nodeName;
                    this.setChildren(new TemplateChildren(n));
                    return;
                }
            }
            Node p = getParentNode();
            if (p != null) {
                TemplateChildren ch = (TemplateChildren)p.getChildren();
                if (ch != null) {
                    ch.refreshKeyImpl(n);
                }
            }
        }

        @Override
        public void childrenAdded(NodeMemberEvent ev) {
            refresh();
        }

        @Override
        public void childrenRemoved(NodeMemberEvent ev) {
            refresh();
        }

        @Override
        public void childrenReordered(NodeReorderEvent ev) {
        }

        @Override
        public void nodeDestroyed(NodeEvent ev) {
        }

        @Override
        public void propertyChange(PropertyChangeEvent evt) {
            refresh();
        }
    }

    private static class DefaultTemplateWizard extends TemplateWizard {
        DefaultTemplateWizard() {}
    }
    
    // delegate action
    // bugfix 36573, NewTemplateInlineAction provides context aware action
    private static final Node[] EMPTY_NODE_ARRAY = new Node[0];
    
    private class NodeLookupListener implements LookupListener {
        
        public void resultChanged (org.openide.util.LookupEvent ev) {
            updateAction ();
        }
    }
    
    private void updateAction () {}
    
    /*STARBEANS - extended to global context and made protected*/
    static protected final synchronized Node[] getNodesFromLookup (Lookup lookup) {
        if(lookup==null) {
            lookup = Utilities.actionsGlobalContext();
        }
        if (lookup != null) {
            return lookup.lookupAll(Node.class).toArray(EMPTY_NODE_ARRAY);
        } 
        return EMPTY_NODE_ARRAY;
    }
    
    
    /** Implements <code>ContextAwareAction</code> interface method. */
    @Override
    public Action createContextAwareInstance (Lookup actionContext) {
        return new DelegateAction (this, actionContext);
    }
    
    private static final class DelegateAction extends Object
    implements Action, Presenter.Popup, /*Presenter.Menu, Presenter.Toolbar,*/ LookupListener {
        
        private NewTemplateInlineAction delegate;
        private Lookup actionContext;
        private Lookup.Result nodesResult;
        
        private PropertyChangeSupport support = new PropertyChangeSupport (this);
        
        public DelegateAction (NewTemplateInlineAction action, Lookup actionContext) {
            this.delegate = action;
            this.actionContext = actionContext;
            this.nodesResult = actionContext.lookupResult(Node.class);
            // if a weak listener is used then NewTemplateActionTest fails
            //LookupListener l = (LookupListener)WeakListeners.create (LookupListener.class, (LookupListener)action, nodesResult);
            //nodesResult.addLookupListener (l);
            //l.resultChanged (null);
            nodesResult.addLookupListener (this);
            resultChanged (null);
        }
        
        /** Overrides superclass method, adds delegate description. */
        @Override
        public String toString () {
            return super.toString () + "[delegate=" + delegate + "]"; // NOI18N
        }
        
        public void putValue (String key, Object value) { }
        
        public boolean isEnabled () {
            return delegate.enable (getNodesFromLookup (actionContext));
        }
        
        public Object getValue (String key) {
            return delegate.getValue (key);
        }
        
        public void setEnabled (boolean b) {
        }
        
        public void actionPerformed (ActionEvent e) {
        }
        
        public void addPropertyChangeListener (PropertyChangeListener listener) {
            support.addPropertyChangeListener (listener);
        }
        
        public void removePropertyChangeListener (PropertyChangeListener listener) {
            support.removePropertyChangeListener (listener);
        }
        
        public JMenuItem getPopupPresenter() {
            return delegate.getPopupPresenter (actionContext, this);
        }
    
        public void resultChanged (org.openide.util.LookupEvent ev) {
            getPopupPresenter ();
//            getMenuPresenter ();
//            getToolbarPresenter ();
        }
        
//        public JMenuItem getMenuPresenter () {
//            return delegate.getMenuPresenter ();
//        }
//        
//        public Component getToolbarPresenter () {
//            return delegate.getToolbarPresenter ();
//        }
//        
    }
}
