/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2009 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or sbbuild/licenses/cddl-gpl-2-cp.txt. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * sbbuild/licenses/cddl-gpl-2-cp.txt.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2009 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 *
 * Portions Copyrighted 2008-2014 George Waldon
 */

package org.starbeans.modules.corelite.contextmenus;

import java.util.ResourceBundle;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.Action;
import org.openide.NotifyDescriptor;
import org.openide.actions.*;
import org.openide.cookies.InstanceCookie;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataFolder;
import org.openide.loaders.DataObject;
import org.openide.loaders.InstanceDataObject;
import org.openide.nodes.FilterNode;
import org.openide.nodes.Node;
import org.openide.nodes.PropertySupport;
import org.openide.nodes.Sheet;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;
import org.openide.util.actions.SystemAction;
import org.openide.util.datatransfer.NewType;

/** Node for the Context2Action action representation.
 *
 * @author George Waldon
 */
public class Context2ActionNode extends DataFolder.FolderNode {

    private static final Logger LOGGER = Logger.getLogger(Context2ActionNode.class.getName());

    /** Actions supported by the nodes */
    static SystemAction[] actionslv1;
    static SystemAction[] actionslv2;
    static SystemAction[] actionslv3;
    private static final Node.PropertySet[] NODE_PROPERTIES = new Node.PropertySet[0];
    private static final ResourceBundle bundle = NbBundle.getBundle(Context2ActionNode.class);
    private DataFolder folder;
    /** Measure the folder depth in the FileSystem */
    private int fddepth;

    public Context2ActionNode() {
        this(Context2Manager.findContext2Folder());
    }

    private Context2ActionNode(DataFolder folder) {
        folder. super(
          
        new C2ActionChildren(folder));
        this.folder = folder;
        super.setShortDescription(bundle.getString("CTL_C2Node_hint"));  //NOI18N
        FileObject fo = folder.getPrimaryFile();
        while ((fo = fo.getParent()) != null) {
            fddepth++;
        }
    }

    @Override
    public HelpCtx getHelpCtx() {
        return new HelpCtx(Context2ActionNode.class);
    }

    /** Actions for this node which depend on folder depth.
     *
     * @return array of actions for this node
     */
    @Override
    public SystemAction[] getActions() {
        if (fddepth < 2) {
            if (actionslv1 == null) {
                actionslv1 = new SystemAction[]{
                    SystemAction.get(FileSystemAction.class),
                    null,
                    SystemAction.get(ToolsAction.class),
                    SystemAction.get(PropertiesAction.class),
                };
            }
            return actionslv1;
        } else if (fddepth == 2) {
            if (actionslv2 == null) {
                actionslv2 = new SystemAction[]{
                    SystemAction.get(FileSystemAction.class),
                    null,
                    SystemAction.get(ReorderAction.class),
                    null,
                    SystemAction.get(CutAction.class),
                    SystemAction.get(CopyAction.class),
                    SystemAction.get(PasteAction.class),
                    null,
                    SystemAction.get(DeleteAction.class),
                    SystemAction.get(RenameAction.class),
                    null,
                    SystemAction.get(NewAction.class),
                    null,
                    SystemAction.get(ToolsAction.class),
                    SystemAction.get(PropertiesAction.class),
                };
            }
            return actionslv2;
        } else {
            if (actionslv3 == null) {
                actionslv3 = new SystemAction[]{
                    SystemAction.get(FileSystemAction.class),
                    null,
                    SystemAction.get(MoveUpAction.class),
                    SystemAction.get(MoveDownAction.class),
                    SystemAction.get(ReorderAction.class),
                    null,
                    SystemAction.get(CutAction.class),
                    SystemAction.get(CopyAction.class),
                    SystemAction.get(PasteAction.class),
                    null,
                    SystemAction.get(DeleteAction.class),
                    SystemAction.get(RenameAction.class),
                    null,
                    SystemAction.get(NewAction.class),
                    null,
                    SystemAction.get(ToolsAction.class),
                    SystemAction.get(PropertiesAction.class),
                };
            }
            return actionslv3;
        }
    }

    @Override
    public boolean canDestroy() {
        return fddepth > 2;
    }

    @Override
    public boolean canCut() {
        return fddepth > 2;
    }

    @Override
    public boolean canCopy() {
        return fddepth > 2;
    }

    @Override
    public boolean canRename() {
        return fddepth > 1;
    }

    /** Creates properties for this node.
     */
    @Override
    public Node.PropertySet[] getPropertySets() {
        if (fddepth < 2) {
            return NODE_PROPERTIES;
        } else {
            Sheet sheet = Sheet.createDefault();
            sheet.get(Sheet.PROPERTIES).put(
                    new PropertySupport.Name(
                    this,
                    bundle.getString("PROP_MenuName"),
                    bundle.getString("HINT_MenuName")
                    ));
            return sheet.toArray();
        }
    }

    /** Support for new types.
     * 
     */
    @Override
    public NewType[] getNewTypes() {
        NewType newMenu = new NewType() {

            @Override
            public String getName() {
                return bundle.getString("CTL_newMenuName");
            }

            @Override
            public void create() throws java.io.IOException {
                addMenu();
            }
        };

        return new NewType[]{
            newMenu,
            new NewType() {

            @Override
                public String getName() {
                    return bundle.getString("CTL_newMenuSeparator");
                }

            @Override
                public void create() throws java.io.IOException {
                    addSeparator();
                }
            }
        };
    }

    void addMenu() {
        NotifyDescriptor.InputLine des = new NotifyDescriptor.InputLine(bundle.getString("LBL_MenuLabel"),
                bundle.getString("LBL_MenuDialog"));
        des.setInputText(bundle.getString("LBL_MenuInput"));

        Object ob = org.openide.DialogDisplayer.getDefault().notify(des);
        if (ob == NotifyDescriptor.OK_OPTION) {
            String s = des.getInputText();
            if (!s.equals("")) { // NOI18N
                FileObject fo = folder.getPrimaryFile();
                try {
                    FileObject newfo = fo.getFileObject(s);
                    if (newfo == null) {
                        String lastItemName = getLastItemName();
                        newfo = fo.createFolder(s);
                        if (lastItemName != null) {
                            fo.setAttribute( //ordering
                                    lastItemName + "/" + newfo.getNameExt(), // NOI18N
                                    Boolean.TRUE);
                        }
                    }
                } catch (java.io.IOException e) {
                    LOGGER.log(Level.FINE,"",e);
                }
            }
        }
    }

    void addSeparator() {
        try {
            InstanceDataObject insdata = InstanceDataObject.find(folder, null, "javax.swing.JSeparator"); // NOI18N            
            String lastName = getLastItemName();
            DataObject doj;

            if (insdata == null) {
                doj = InstanceDataObject.create //create separator
                        (folder, null, "javax.swing.JSeparator"); // NOI18N
            } else {
                doj = insdata.copy(folder); //copy separator
            }
            if (lastName != null) {
                folder.getPrimaryFile().setAttribute( //ordering
                        lastName + "/" + doj.getPrimaryFile().getNameExt(), // NOI18N
                        Boolean.TRUE);
            }
        } catch (java.io.IOException e) {
            LOGGER.log(Level.FINE,"",e);
        }
    }

    /** 
     * @return name of last menu or <code>null</code> if there is no one 
     */
    private String getLastItemName() {
        String lastItemName = null;
        Node[] n = getChildren().getNodes();
        if (n.length > 0) {
            Node last = n[n.length - 1];
            DataObject d = last.getCookie(DataObject.class);
            if (d != null) {
                lastItemName = d.getPrimaryFile().getNameExt();
            }
        }

        return lastItemName;
    }

    /** Children for the Context2Action Node. Creates Context2ActionNode or
     * ActionNode as filter subnodes...
     */
    static final class C2ActionChildren extends FilterNode.Children {

        private String folderName = null;

        /** @param node to take children from */
        public C2ActionChildren(DataFolder folder) {
            super(folder.getNodeDelegate());
            folderName = folder.getName();
        }

        /** Prevent Context2Actions to show up in the tree.
         **/
        @Override
        protected Node[] createNodes(Node key) {
            Node n = key;
            InstanceCookie cookie = n.getCookie(InstanceCookie.class);
            if (cookie instanceof InstanceCookie.Of) {
                if (((InstanceCookie.Of) cookie).instanceOf(Context2Action.class)) {
                    try {
                        final Object o = cookie.instanceCreate();
                        String name = ((Context2Action) o).getFolderName();
                        if (name.equals(folderName)) {
                            return null;
                        }
                    } catch (java.io.IOException ex) {
                        LOGGER.log(Level.INFO,"",ex);
                    } catch (ClassNotFoundException ex) {
                        LOGGER.log(Level.INFO,"",ex);
                    }
                }
            }
            return super.createNodes(key);
        }

        /** Overriden, see <code>FilterNode.Children</code> javadoc.
         *
         * @param node node to create copy of
         * @return Context2ActionNode if child node is folder or
         * ActionNode if is action.
         */
        @Override
        protected Node copyNode(Node node) {
            DataFolder df = node.getCookie(DataFolder.class);
            if (df != null) {
                return new Context2ActionNode(df);
            }
            return new ActionNode(node);
        }
    }

    static final class ActionNode extends FilterNode {

        /** Actions supported by this node */
        //static SystemAction[] stdactions;
        static Action[] stdactions;

        ActionNode(Node filter) {
            super(filter, Children.LEAF);
        }

        @Override
        public javax.swing.Action[] getActions(boolean context) {
            if (stdactions == null) {
                stdactions = new Action[]{
                    SystemAction.get(MoveUpAction.class),
                    SystemAction.get(MoveDownAction.class),
                    null,
                    SystemAction.get(CutAction.class),
                    SystemAction.get(CopyAction.class),
                    null,
                    SystemAction.get(DeleteAction.class),
                    null,
                    SystemAction.get(ToolsAction.class),
                    SystemAction.get(PropertiesAction.class),
                };
            }
            return stdactions;
        }

        @Override
        public boolean canRename() {
            return false;
        }

        @Override
        public Node.PropertySet[] getPropertySets() {
            return NODE_PROPERTIES;
        }
    }
}
